/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.07.2017  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include <stdio.h>
#include "project.h"
#include "hardware.h"

#include "system.h"
#include "sys/alt_irq.h"
#include "sys/alt_stdio.h"
#include "altera_avalon_pio_regs.h"
#include "altera_avalon_timer_regs.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static volatile uint32_t dSystemTick = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  SysTickHandler                                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SysTickHandler (void* context)
{
   /* Clear the interrupt */
   IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
   
   /* 
    * Dummy read to ensure IRQ is negated before the ISR returns.
    * The control register is read because reading the status
    * register has side-effects per the register map documentation.
    */
   IORD_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE);

   /* Handle SysTick counter */   
   dSystemTick++;
    
} /* SysTickHandler */

/*************************************************************************/
/*  InitSysTick                                                          */
/*                                                                       */
/*  Init the SysTick timer.                                              */
/*                                                                       */                                                                       
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitSysTick (void)
{
   alt_irq_context   status;
   static uint8_t   bInitDone = 0;
   uint32_t         dHirResPeriod;
   uint16_t         wValue;
   
   if (0 == bInitDone)
   {
      bInitDone = 1;

      /* Calculate timer period value */
      dHirResPeriod = (SYSTICK_FREQ / TICKS_PER_SECOND) - 1;
      
      /* Set period register */
      IOWR_ALTERA_AVALON_TIMER_PERIODH(SYSTICK_BASE, ((dHirResPeriod>>16) & 0xFFFF));      
      IOWR_ALTERA_AVALON_TIMER_PERIODL(SYSTICK_BASE, (dHirResPeriod & 0xFFFF));

      /* Set control register, stop timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_STOP_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);
      
      /* Clear status */
      IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
  
      /* Set SysTick interrupt handler */
      status = alt_irq_disable_all();  
      alt_ic_isr_register(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ, SysTickHandler, 0x0, 0x0);
      alt_ic_irq_disable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
      alt_irq_enable_all(status);

      /* Set control register, start timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_START_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK  |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);

      /* Enable SysTick interrupt */   
      alt_ic_irq_enable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
   }      
} /* InitSysTick */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  OSTimeDly                                                            */
/*                                                                       */
/*  Loop for a specified number of milliseconds,                         */
/*  will not release the CPU.                                            */
/*                                                                       */
/*  In    : TimeoutMs                                                    */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OSTimeDly (uint32_t dTimeoutMs)
{
   uint32_t dTicks;   
   uint32_t dTimeEnd;
   
   dTicks   = OS_MS_2_TICKS(dTimeoutMs);
   dTimeEnd = dSystemTick + dTicks;
   
   while (dSystemTick <= dTimeEnd)
   {
      __asm__ volatile ("nop");
   }
      
} /* OSTimeDly */

/*************************************************************************/
/*  OSTimeGet                                                            */
/*                                                                       */
/*  Return the time from the system ticker.                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Systick                                                      */
/*************************************************************************/
uint32_t OSTimeGet (void)
{
   return(dSystemTick);
} /* OSTimeGet */

/*************************************************************************/
/*  hw_TimerInit                                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void hw_TimerInit (void)
{
   /* Init SysTick */
   InitSysTick();

} /* hw_TimerInit */

/*************************************************************************/
/*  hw_LED                                                               */
/*                                                                       */
/*  Write to the LED port.                                               */
/*                                                                       */
/*  In    : dValue                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void hw_LED (uint32_t dValue)
{
   IOWR_ALTERA_AVALON_PIO_DATA(PIO_LED_BASE, dValue);
} /* hw_LED */

/*** EOF ***/
