/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.07.2017  mifi  First Version.
*                    This bootloader is based on the Altera example
*                    "C Booloader" which can be find here:
*                    http://www.alterawiki.com/wiki/EPCS_bootloaders
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include "project.h"
#include "hardware.h"
#include "xboot.h"

#include "system.h"
#include "io.h"
#include "altera_avalon_spi.h"
#include "epcs_commands.h"
#include "sys/alt_irq.h"
#include "sys/alt_cache.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  OutputBootMessage                                                    */
/*                                                                       */
/*  Output boot message.                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputBootMessage (void)
{
   alt_printf("\r\n");
   alt_printf("*********************************\r\n");
   alt_printf("  Project: %s\r\n", PROJECT_NAME);
   alt_printf("  Board  : %s\r\n", PROJECT_BOARD);
   alt_printf("  Version: v%s\r\n", PROJECT_VER_STRING);
   alt_printf("  Build  : "__DATE__ " " __TIME__"\r\n");
   alt_printf("*********************************\r\n");
   alt_printf("\r\n");
   
} /* OutputBootMessage */

/*************************************************************************/
/*  epcs_read_buffer                                                     */
/*                                                                       */
/*  Read EPCS data to the given buffer.                                  */
/*                                                                       */
/*  In    : base, offset, dest_addr, length                              */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
alt_32 epcs_read_buffer (alt_u32 base, int offset, alt_u8 *dest_addr, int length)
{
  alt_u8 read_command[4];

  read_command[0] = 0x03;
  read_command[1] = (offset >> 16) & 0xFF;
  read_command[2] = (offset >> 8) & 0xFF;
  read_command[3] = offset & 0xFF;

  alt_avalon_spi_command(
    base,
    0,
    sizeof(read_command) / sizeof(*read_command),
    read_command,
    length,
    (alt_u8*)dest_addr,
    0
  );

  return length;
} /* epcs_read_buffer */

/*************************************************************************/
/*  CopyAndStartFirmware                                                 */
/*                                                                       */
/*  Copy and start the firmware.                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: only in case of an error                                     */
/*************************************************************************/
static void CopyAndStartFirmware (void)
{
   uint32_t dBaseAddress;
   uint32_t dDataLength;
   uint32_t dDestAddress;
   uint32_t dOffset;

   /* Base Address of EPCS memory Area */
   dBaseAddress = EPCS_FLASH_CONTROLLER_BASE + EPCS_FLASH_CONTROLLER_REGISTER_OFFSET ;
   
#if defined(XBOOT_MODE)   
   alt_printf("--- XBOOT Mode ---\r\n\r\n");
#endif   
    
   /* Get the EPCS reset vector offset */    
   dOffset = IORD(EPCS_RESET_OFFSET_BASE, 0);

#if defined(XBOOT_MODE)   
   alt_printf("Checking code at 0x%x + 0x%x\r\n", dOffset, sizeof(XBOOT_HEADER));
   dOffset += sizeof(XBOOT_HEADER);
#else   
   alt_printf("Checking code at 0x%x\r\n", dOffset);
#endif   
    
   /*
    * Copy the payload from the EPCS flash to the memory
    */ 

   /* Read data size and destination address from the first header */
   epcs_read_buffer(dBaseAddress, dOffset, (uint8_t*)&dDataLength, sizeof(dDataLength));
   dOffset = dOffset + 4;
   epcs_read_buffer(dBaseAddress, dOffset, (uint8_t*)&dDestAddress, sizeof(dDestAddress));
   dOffset = dOffset + 4;
   
   while(dDataLength)
   {
      if(0xffffffff == dDataLength)
      {
         alt_printf("Out of range 0x%x bytes at address 0x%x\r\n", dDataLength, dDestAddress);
         break;
      }
      
      /* Copy from the EPCS flash to memory */
      alt_printf("Copying 0x%x bytes to address 0x%x\r\n", dDataLength, dDestAddress);
       
      epcs_read_buffer(dBaseAddress, dOffset, (uint8_t*)dDestAddress, dDataLength);
      dOffset = dOffset + dDataLength;
      
      /* Read data size and destination address from the next header */
      epcs_read_buffer(dBaseAddress, dOffset, (uint8_t*)&dDataLength, sizeof(dDataLength));
      dOffset = dOffset + 4;
      epcs_read_buffer(dBaseAddress, dOffset, (uint8_t*)&dDestAddress, sizeof(dDestAddress));
      dOffset = dOffset + 4;
   }
   
   /* if length is 0 then this is the start address of the firmware */
   if(0 == dDataLength)
   {
      alt_printf("\r\nStarting firmware at 0x%x\r\n\r\n", dDestAddress);
      
      alt_irq_disable_all();
      alt_icache_flush_all();
      alt_dcache_flush_all();
      asm volatile ( "callr %0" ::"r"(dDestAddress) );   /* Will not come back */
   }

   alt_printf("\r\nError, no valid firmware found\r\n\r\n");

} /* CopyAndStartFirmware */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   hw_LED(0);
   OutputBootMessage();
   
   CopyAndStartFirmware();

   /*
    * In case of an ERROR, the next code will be executed
    */

   hw_TimerInit();
   
   while (1)
   {
      hw_LED(3);
      OSTimeDly(100);
      hw_LED(0);
      OSTimeDly(100);
   }

  /*
   * This return here make no sense.
   * But to prevent the compiler warning:
   * "return type of 'main' is not 'int'
   * we use an int as return :-)
   */ 
  return(0);
} /* main */

/*** EOF ***/
